print("import: 'binstar_client'")
import binstar_client

print("import: 'binstar_client.commands'")
import binstar_client.commands

print("import: 'binstar_client.inspect_package'")
import binstar_client.inspect_package

print("import: 'binstar_client.inspect_package.tests'")
import binstar_client.inspect_package.tests

print("import: 'binstar_client.mixins'")
import binstar_client.mixins

print("import: 'binstar_client.scripts'")
import binstar_client.scripts

print("import: 'binstar_client.tests'")
import binstar_client.tests

print("import: 'binstar_client.utils'")
import binstar_client.utils

print("import: 'binstar_client.utils.notebook'")
import binstar_client.utils.notebook

print("import: 'binstar_client.utils.notebook.tests'")
import binstar_client.utils.notebook.tests




"""
Log out from binstar
"""
from binstar_client.utils import get_server_api, remove_token

import logging
from binstar_client import errors

logger = logging.getLogger('binstar.logout')


def main(args):

    aserver_api = get_server_api(args.token, args.site)
    if aserver_api.token:
    # TODO: named 'application' because I was using the github model
    # Change it to name once we release the latest version of binstar server
        try:
            aserver_api.remove_authentication()
        except errors.Unauthorized as err:
            logger.debug("The token that you are trying to remove may not be valid"
                      "{}".format(err))

        remove_token(args)
        logger.info("logout successful")
    else:
        logger.info("You are not logged in")


def add_parser(subparsers):
    subparser = subparsers.add_parser('logout',
                                      help='Log out from your Anaconda repository',
                                      description=__doc__)

    subparser.set_defaults(main=main)



from __future__ import print_function

import logging
import os
import shutil
import sys
import tempfile

from binstar_client import errors

logger = logging.getLogger('binstar.projects.upload')


class _TmpDir(object):
    def __init__(self, prefix):
        self._dir = tempfile.mkdtemp(prefix=prefix)

    def __exit__(self, type, value, traceback):
        try:
            shutil.rmtree(path=self._dir)
        except Exception as e:
            # prefer original exception to rmtree exception
            if value is None:
                print("Exception cleaning up TmpDir %s: %s" % (self._dir, str(e)), file=sys.stderr)
                raise e
            else:
                print("Failed to clean up TmpDir %s: %s" % (self._dir, str(e)), file=sys.stderr)
                raise value

    def __enter__(self):
        return self._dir


def _real_upload_project(project, args, username):
    try:
        from anaconda_project import project_ops
    except ImportError:
        raise errors.BinstarError('anaconda-project package is not installed')

    print("Uploading project: {}".format(project.name))

    status = project_ops.upload(
        project, site=args.site, username=username, token=args.token, log_level=args.log_level,
    )

    if not status:
        for error in status.errors:
            print(error, file=sys.stderr)
        print(status.status_description, file=sys.stderr)
        raise errors.BinstarError(status.status_description)

    print(status.status_description)
    return [project.name, status.url]


def upload_project(project_path, args, username):
    try:
        from anaconda_project import project
        from anaconda_project import project_ops
    except ImportError:
        raise errors.BinstarError(
            'To upload projects such as {}, install the anaconda-project package.'.format(project_path),
        )

    if os.path.exists(project_path) and not os.path.isdir(project_path):
        # make the single file into a temporary project directory
        with (_TmpDir(prefix="anaconda_upload_")) as dirname:
            shutil.copy(project_path, dirname)
            basename_no_extension = os.path.splitext(os.path.basename(project_path))[0]
            new_project = project_ops.create(dirname, name=basename_no_extension)
            return _real_upload_project(new_project, args, username)
    else:
        new_project = project.Project(directory_path=project_path)
        return _real_upload_project(new_project, args, username)



import fnmatch
import logging

from os.path import exists, isfile, join

logger = logging.getLogger('binstar.projects.upload')


class NoIgnoreFileException(IOError):
    def __init__(self, msg, *args, **kwargs):
        self.msg = msg
        super(NoIgnoreFileException, self).__init__(msg, *args, **kwargs)


class FilterBase(object):
    def __init__(self, *args, **kwargs):
        raise NotImplemented

    def can_filter(cls):
        return True

    def run(self, pfile):
        raise NotImplemented

    def update_metadata(self, metadata={}):
        raise NotImplemented


class VCSFilter(FilterBase):
    '''
    Version Control System Filtering
    '''
    def __init__(self, pfiles, *args, **kwargs):
        self.pfiles = pfiles

    def run(self, pfile):
        if pfile.relativepath.startswith('.git/'):
            return False
        if pfile.relativepath.startswith('.svn/'):
            return False
        if pfile.relativepath.startswith('.hg/'):
            return False
        if pfile.relativepath.startswith('.anaconda/'):
            return False
        return True


class FilesFilter(FilterBase):
    '''
    Ignore specific files
    '''
    ignored = ['.anaconda/project-local.yml', '.anaconda/project-local.yaml']

    def __init__(self, pfiles, *args, **kwargs):
        self.pfiles = pfiles

    def run(self, pfile):
        return pfile.relativepath not in self.ignored


class LargeFilesFilter(FilterBase):
    max_file_size = 2097152

    def __init__(self, pfiles, *args, **kwargs):
        self.pfiles = pfiles

    def run(self, pfile):
        if pfile.size > self.max_file_size:
            return False
        return True


class ProjectIgnoreFilter(FilterBase):
    def __init__(self, pfiles, *args, **kwargs):
        self._patterns = None
        self.pfiles = pfiles
        self.basepath = kwargs.get('basepath', '.')

    @property
    def patterns(self):
        if self._patterns is None:
            try:
                self._patterns = ignore_patterns(self.basepath)
            except NoIgnoreFileException:
                logger.debug("No ignore file")
        return self._patterns

    def can_filter(self):
        return self.patterns is not None and len(self.patterns) > 0

    def run(self, pfile):
        for pattern in self.patterns:
            if fnmatch.fnmatch(pfile.relativepath, pattern):
                return False
            elif fnmatch.fnmatch(pfile.relativepath.split('/')[0], pattern):
                return False
        return True


def get_ignore_file(basepath):
    ignore_files = ['.projectignore', '.gitignore']
    for ignore_file in ignore_files:
        ignore_file_path = join(basepath, ignore_file)
        if exists(ignore_file_path) and isfile(ignore_file_path):
            logger.debug("Ignore patterns file: {}".format(ignore_file_path))
            return ignore_file_path
    raise NoIgnoreFileException("There is no .projectignore or .gitignore")


def ignore_patterns(basepath):
    patterns = []
    with open(get_ignore_file(basepath)) as ifile:
        for row in ifile:
            pattern = remove_comments(clean(row))
            patterns.append(pattern)
    return patterns


def clean(cad):
    return cad.strip()


def remove_comments(cad):
    return cad.split("#", 1)[0].strip()

filters = [VCSFilter, ProjectIgnoreFilter, LargeFilesFilter]
